searchData={"items":[{"type":"module","title":"ExMP4","doc":"# ExMP4\n\n[![Hex.pm](https://img.shields.io/hexpm/v/ex_mp4.svg)](https://hex.pm/packages/ex_mp4)\n[![API Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/ex_mp4)\n\n`ISO-MP4` reader and writer.\n\nThis package contains MPEG-4 specifications defined in parts:\n* ISO/IEC 14496-12 - ISO Base Media File Format (QuickTime, MPEG-4, etc)\n* ISO/IEC 14496-14 - MP4 file format\n\nThis package is an alternative to and inspired from [membrane_mp4_plugin](https://github.com/membraneframework/membrane_mp4_plugin). This package differs from the `membrane_mp4_plugin` in that it allows to manipulate mp4 files without the usage of membrane pipelines.","ref":"ExMP4.html"},{"type":"module","title":"Installation - ExMP4","doc":"The package can be installed by adding `ex_mp4` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ex_mp4, \"~> 0.4.0\"}\n  ]\nend\n```","ref":"ExMP4.html#module-installation"},{"type":"module","title":"API - ExMP4","doc":"The API is not yet stable, so breaking changes may occur when upgrading minor version.","ref":"ExMP4.html#module-api"},{"type":"module","title":"Usage - ExMP4","doc":"check the `examples` folder for usage.","ref":"ExMP4.html#module-usage"},{"type":"function","title":"ExMP4.base_date/0","doc":"","ref":"ExMP4.html#base_date/0"},{"type":"function","title":"ExMP4.movie_timescale/0","doc":"","ref":"ExMP4.html#movie_timescale/0"},{"type":"type","title":"ExMP4.duration/0","doc":"","ref":"ExMP4.html#t:duration/0"},{"type":"type","title":"ExMP4.offset/0","doc":"","ref":"ExMP4.html#t:offset/0"},{"type":"type","title":"ExMP4.timescale/0","doc":"","ref":"ExMP4.html#t:timescale/0"},{"type":"module","title":"ExMP4.FWriter","doc":"Module responsible for writing fragmented MP4.","ref":"ExMP4.FWriter.html"},{"type":"function","title":"ExMP4.FWriter.close/1","doc":"Close the writer.","ref":"ExMP4.FWriter.html#close/1"},{"type":"function","title":"ExMP4.FWriter.create_fragment/1","doc":"Create a new empty fragment.\n\nAfter adding samples, the fragment should be flashed, with `flush_fragment/1`.","ref":"ExMP4.FWriter.html#create_fragment/1"},{"type":"function","title":"ExMP4.FWriter.flush_fragment/1","doc":"Flush the current fragment.","ref":"ExMP4.FWriter.html#flush_fragment/1"},{"type":"function","title":"ExMP4.FWriter.new/4","doc":"Create a new mp4 writer that writes to filesystem.\n\nThe tracks are assigned an id starting from 1.\n\nThe following options can be provided:\n  * `major_brand` - Set the major brand\n  * `compatible_brands` - Set the compatible brands\n  * `creation_time` - Set the creation time\n  * `modification_time` - Set the modification time\n  * `duration` - Set the total duration if known. The value can be `true`, `false` or an integer.\n\n    If `true`, the total duration of the presentation is calculated when closing the `writer` and the `mehd` box is\n    set to include the fragment duration. Note that this needs the output target to support seeking (not suitable for live streaming.)\n\n    If `false`, the total duration is not calculated and the `mehd` box is not included. This is suitable for real time or\n    for presentations where the total duration is not available.\n\n    If an integer, it's the total duration in the `movie` timescale and it'll be set in the `mehd` box.\n\nThe last argument is an optional module implementing `ExMP4.FragDataWriter`.","ref":"ExMP4.FWriter.html#new/4"},{"type":"function","title":"ExMP4.FWriter.new!/4","doc":"The same as `new/2`, but raises if it fails.","ref":"ExMP4.FWriter.html#new!/4"},{"type":"function","title":"ExMP4.FWriter.write_sample/2","doc":"Write a sample to the current fragment.","ref":"ExMP4.FWriter.html#write_sample/2"},{"type":"type","title":"ExMP4.FWriter.new_opts/0","doc":"Options to supply when creating the writer.","ref":"ExMP4.FWriter.html#t:new_opts/0"},{"type":"type","title":"ExMP4.FWriter.t/0","doc":"","ref":"ExMP4.FWriter.html#t:t/0"},{"type":"module","title":"ExMP4.Helper","doc":"Helper functions.","ref":"ExMP4.Helper.html"},{"type":"function","title":"ExMP4.Helper.timescalify/3","doc":"Convert duration between different timescales.","ref":"ExMP4.Helper.html#timescalify/3"},{"type":"type","title":"ExMP4.Helper.timescale/0","doc":"","ref":"ExMP4.Helper.html#t:timescale/0"},{"type":"module","title":"ExMP4.Reader","doc":"This module contains function to read mp4 sources.\n\nThe following fields are public:\n\n  * `duration` - The duration of the mp4 mapped in `:timescale` unit.\n  * `timescale` - The timescale of the mp4.\n  * `fragmented?` - The MP4 file is fragmented.\n  * `creation_time` - Creation date time of the presentation.\n  * `modification_time` - Modification date time of the presentation.\n  * `major_brand`\n  * `major_brand_version`\n  * `compatible_brands`","ref":"ExMP4.Reader.html"},{"type":"module","title":"Examples - ExMP4.Reader","doc":"# Read a file\n    {:ok, reader} = ExMP4.Reader.new(\"some_mp4_file.mp4\")\n\n    IO.inspect(\"Duration: #{reader.duration}\")\n    IO.inspect(\"Timescale: #{reader.timescale}\")\n    IO.inspect(\"Major Brand: #{reader.major_brand}\")\n\n    # Get tracks information\n    for track <- ExMP4.Reader.tracks(reader) do\n      IO.inspect(\"Track information :\")\n      IO.inspect(\"====================\")\n      IO.inspect(\"  Id: #{track.id}\")\n      IO.inspect(\"  Type: #{track.type}\")\n      IO.inspect(\"  Media: #{track.media}\")\n      IO.inspect(\"  Duration: #{track.duration}\")\n      IO.inspect(\"  Timescale: #{track.timescale}\")\n      IO.inspect(\"\")\n    end\n\n    # Read samples from the first track\n    track = ExMP4.Reader.tracks(reader) |> List.first()\n    Enum.each(0..(track.sample_count - 1), fn sample_id ->\n      sample = ExMP4.Reader.read_sample(reader, track.id, sample_id)\n      IO.inspect(\"Size of the sample: #{byte_size(sample.content)}\")\n    end)","ref":"ExMP4.Reader.html#module-examples"},{"type":"module","title":"MP4 Source - ExMP4.Reader","doc":"In the last example, the source of the mp4 is a file, to read from other sources (binary, http server, ...etc.), you need to write\na module that implements the `ExMP4.Read` behaviour.","ref":"ExMP4.Reader.html#module-mp4-source"},{"type":"function","title":"ExMP4.Reader.close/1","doc":"Close the reader and free resources.","ref":"ExMP4.Reader.html#close/1"},{"type":"function","title":"ExMP4.Reader.duration/2","doc":"Get the duration of the stream.","ref":"ExMP4.Reader.html#duration/2"},{"type":"function","title":"ExMP4.Reader.new/2","doc":"Create a new MP4 reader.\n\nThe input may be a file name, a binary `{:binary, data}` or any other input with\nan module implementing `ExMP4.DataReader` behaviour.","ref":"ExMP4.Reader.html#new/2"},{"type":"function","title":"ExMP4.Reader.new!/2","doc":"The same as `new/1`, but raises if it fails.","ref":"ExMP4.Reader.html#new!/2"},{"type":"function","title":"ExMP4.Reader.read_sample/3","doc":"Read a sample from the specified track.\n\nThe first `sample_id` of any track starts at `0`. The `sample_count` field\nof track provides the total number of samples on the track.\n\nRetrieving samples by their id is slow since it scans all the metadata to get\nthe specified sample, a better approach is to stream all the samples using `stream/2`.","ref":"ExMP4.Reader.html#read_sample/3"},{"type":"function","title":"ExMP4.Reader.samples/2","doc":"Get samples.","ref":"ExMP4.Reader.html#samples/2"},{"type":"function","title":"ExMP4.Reader.stream/2","doc":"Stream the samples' metadata.\n\nThe samples are retrieved ordered by their `dts` value.","ref":"ExMP4.Reader.html#stream/2"},{"type":"function","title":"ExMP4.Reader.tracks/1","doc":"Get all the available tracks.","ref":"ExMP4.Reader.html#tracks/1"},{"type":"type","title":"ExMP4.Reader.stream_opts/0","doc":"Stream options.\n\n  - `tracks` - stream only the specified tracks.","ref":"ExMP4.Reader.html#t:stream_opts/0"},{"type":"type","title":"ExMP4.Reader.t/0","doc":"Struct describing an MP4 reader.","ref":"ExMP4.Reader.html#t:t/0"},{"type":"module","title":"ExMP4.Sample","doc":"A struct describing an MP4 sample (a video frame, an audio sample, ...etc)","ref":"ExMP4.Sample.html"},{"type":"function","title":"ExMP4.Sample.new/1","doc":"Create a new sample","ref":"ExMP4.Sample.html#new/1"},{"type":"type","title":"ExMP4.Sample.id/0","doc":"","ref":"ExMP4.Sample.html#t:id/0"},{"type":"type","title":"ExMP4.Sample.t/0","doc":"","ref":"ExMP4.Sample.html#t:t/0"},{"type":"module","title":"ExMP4.SampleMetadata","doc":"A struct describing the metadata of a sample.","ref":"ExMP4.SampleMetadata.html"},{"type":"type","title":"ExMP4.SampleMetadata.t/0","doc":"Sample metadata struct.\n\n  * `track_id` - The track id of the sample.\n  * `dts` - The decoding time of the sample in track `timescale` units.\n  * `pts` - The presentation (compoistion) time of the sample in track `timescale` units.\n  * `sync?` - Indicates wether the sample is a sync or rap (random access point).\n  * `duration` -  The duration of the sample in track `timescale` units.\n  * `size` - The size of the sample.\n  * `offset` - The offset of the sample in the container.","ref":"ExMP4.SampleMetadata.html#t:t/0"},{"type":"module","title":"ExMP4.Writer","doc":"This module contains functions to write MP4.","ref":"ExMP4.Writer.html"},{"type":"function","title":"ExMP4.Writer.add_track/2","doc":"Add a new track.\n\nA track is created by instantiating the public fields of `ExMP4.Track`. The\nid is assigned by the writer and it's equals to the index of the track starting\nfrom `1`. The first track has an id `1`, the second `2`, ...etc.","ref":"ExMP4.Writer.html#add_track/2"},{"type":"function","title":"ExMP4.Writer.add_tracks/2","doc":"Add multiple tracks.","ref":"ExMP4.Writer.html#add_tracks/2"},{"type":"function","title":"ExMP4.Writer.new/3","doc":"Create a new mp4 writer that writes to filesystem.\n\nThe following options can be provided:\n  * `fast_start` - Move the `moov` box to the beginning of the file. Defaults to: `false`\n\nBy default the writer writes the data to the file system, this behaviour can be changed by\nproviding a module implementing `ExMP4.DataWriter` in the third argument.","ref":"ExMP4.Writer.html#new/3"},{"type":"function","title":"ExMP4.Writer.new!/3","doc":"The same as `new/2`, but raises if it fails.","ref":"ExMP4.Writer.html#new!/3"},{"type":"function","title":"ExMP4.Writer.write_header/2","doc":"Write the mp4 header.\n\nThis function should be called first before adding tracks.","ref":"ExMP4.Writer.html#write_header/2"},{"type":"function","title":"ExMP4.Writer.write_sample/2","doc":"Write a sample.","ref":"ExMP4.Writer.html#write_sample/2"},{"type":"function","title":"ExMP4.Writer.write_trailer/2","doc":"Write the trailer and close the stream.","ref":"ExMP4.Writer.html#write_trailer/2"},{"type":"type","title":"ExMP4.Writer.new_opts/0","doc":"","ref":"ExMP4.Writer.html#t:new_opts/0"},{"type":"type","title":"ExMP4.Writer.t/0","doc":"","ref":"ExMP4.Writer.html#t:t/0"},{"type":"module","title":"ExMP4.Container","doc":"Module for parsing and serializing MP4 files.\n\nBases on MP4 structure specification from `ExMP4.Container.Schema`.","ref":"ExMP4.Container.html"},{"type":"function","title":"ExMP4.Container.box_path/1","doc":"Maps a path in the MP4 box tree into sequence of keys under which that\nbox resides in MP4.","ref":"ExMP4.Container.html#box_path/1"},{"type":"function","title":"ExMP4.Container.get_box/2","doc":"Gets a box from a given path in a parsed MP4.","ref":"ExMP4.Container.html#get_box/2"},{"type":"function","title":"ExMP4.Container.get_box_value/2","doc":"","ref":"ExMP4.Container.html#get_box_value/2"},{"type":"function","title":"ExMP4.Container.parse/1","doc":"Parses binary data to MP4 according to `ExMP4.Container.Schema.schema/0`.\n\nReturns boxes that have been correctly parsed, and the leftover data that\nis insufficient to constitute a complete box.","ref":"ExMP4.Container.html#parse/1"},{"type":"function","title":"ExMP4.Container.parse/2","doc":"Parses binary data to MP4 according to a custom schema.\n\nReturns boxes that have been correctly parsed, and the leftover data that\nis insufficient to constitute a complete box.","ref":"ExMP4.Container.html#parse/2"},{"type":"function","title":"ExMP4.Container.parse!/1","doc":"Same as `parse/1`, raises on error.","ref":"ExMP4.Container.html#parse!/1"},{"type":"function","title":"ExMP4.Container.parse!/2","doc":"Same as `parse/2`, raises on error.","ref":"ExMP4.Container.html#parse!/2"},{"type":"function","title":"ExMP4.Container.serialize/1","doc":"Serializes MP4 to a binary according to `ExMP4.Container.Schema.schema/0`.","ref":"ExMP4.Container.html#serialize/1"},{"type":"function","title":"ExMP4.Container.serialize/2","doc":"Serializes MP4 to a binary according to a custom schema.","ref":"ExMP4.Container.html#serialize/2"},{"type":"function","title":"ExMP4.Container.serialize!/1","doc":"Same as `serialize/1`, raises on error","ref":"ExMP4.Container.html#serialize!/1"},{"type":"function","title":"ExMP4.Container.serialize!/2","doc":"Same as `serialize/2`, raises on error","ref":"ExMP4.Container.html#serialize!/2"},{"type":"function","title":"ExMP4.Container.update_box/4","doc":"Updates a box at a given path in a parsed MP4.\n\nIf `parameter_path` is set, a parameter within a box is updated.","ref":"ExMP4.Container.html#update_box/4"},{"type":"type","title":"ExMP4.Container.box_name_t/0","doc":"","ref":"ExMP4.Container.html#t:box_name_t/0"},{"type":"type","title":"ExMP4.Container.field_name_t/0","doc":"","ref":"ExMP4.Container.html#t:field_name_t/0"},{"type":"type","title":"ExMP4.Container.fields_t/0","doc":"","ref":"ExMP4.Container.html#t:fields_t/0"},{"type":"type","title":"ExMP4.Container.parse_error_context_t/0","doc":"","ref":"ExMP4.Container.html#t:parse_error_context_t/0"},{"type":"type","title":"ExMP4.Container.serialize_error_context_t/0","doc":"","ref":"ExMP4.Container.html#t:serialize_error_context_t/0"},{"type":"type","title":"ExMP4.Container.t/0","doc":"","ref":"ExMP4.Container.html#t:t/0"},{"type":"module","title":"ExMP4.Container.Header","doc":"A structure describing the header of the box.\n\nThe `content_size` field is equal to the box size minus the size of the header (8 bytes).","ref":"ExMP4.Container.Header.html"},{"type":"function","title":"ExMP4.Container.Header.parse/1","doc":"Parses the header of a box.\n\nReturns the `t:t/0` and the leftover data.","ref":"ExMP4.Container.Header.html#parse/1"},{"type":"type","title":"ExMP4.Container.Header.t/0","doc":"","ref":"ExMP4.Container.Header.html#t:t/0"},{"type":"module","title":"ExMP4.Container.Schema","doc":"MP4 structure schema used for parsing and serialization.\n\nUseful resources:\n- https://www.iso.org/standard/79110.html\n- https://www.iso.org/standard/61988.html\n- https://developer.apple.com/library/archive/documentation/QuickTime/QTFF/QTFFChap2/qtff2.html\n- https://github.com/DicomJ/mpeg-isobase/tree/eb09f82ff6e160715dcb34b2bf473330c7695d3b","ref":"ExMP4.Container.Schema.html"},{"type":"function","title":"ExMP4.Container.Schema.schema/0","doc":"Returns `t:ExMP4.Container.Schema.t/0`","ref":"ExMP4.Container.Schema.html#schema/0"},{"type":"type","title":"ExMP4.Container.Schema.field_t/0","doc":"A box field type.\n\nIt may contain a primitive, a list or nested fields. Lists last till the end of a box.","ref":"ExMP4.Container.Schema.html#t:field_t/0"},{"type":"type","title":"ExMP4.Container.Schema.primitive_t/0","doc":"For fields, the following primitive types are supported:\n- `{:int, bit_size}` - a signed integer\n- `{:uint, bit_size}` - an unsigned integer\n- `:bin` - a binary lasting till the end of a box\n- `{:bin, bit_size}` - a binary of given size\n- `:str` - a string terminated with a null byte\n- `{:str, bit_size}` - a string of given size\n- `{:fp, integer_part_bit_size, fractional_part_bit_size}` - a fixed point number","ref":"ExMP4.Container.Schema.html#t:primitive_t/0"},{"type":"type","title":"ExMP4.Container.Schema.schema_def_box_t/0","doc":"","ref":"ExMP4.Container.Schema.html#t:schema_def_box_t/0"},{"type":"type","title":"ExMP4.Container.Schema.schema_def_field_t/0","doc":"","ref":"ExMP4.Container.Schema.html#t:schema_def_field_t/0"},{"type":"type","title":"ExMP4.Container.Schema.schema_def_primitive_t/0","doc":"","ref":"ExMP4.Container.Schema.html#t:schema_def_primitive_t/0"},{"type":"type","title":"ExMP4.Container.Schema.schema_def_t/0","doc":"Type describing the schema definition, that is hardcoded in this module.\n\nIt may be useful for improving the schema definition. The actual schema that\nshould be operated on, or, in other words, the parsed schema definition is\nspecified by `t:ExMP4.Container.Schema.t/0`.\n\nThe schema definition differs from the final schema in the following ways:\n  - primitives along with their parameters are specified as atoms, for example\n  `:int32` instead of `{:int, 32}`\n  - child boxes are nested within their parents directly, instead of residing\n  under `:children` key.\n\nThe schema definition is the following:\n```\n[\n  ftyp: [\n    fields: [\n      major_brand: :str32,\n      major_brand_version: :uint32,\n      compatible_brands: {:list, :str32}\n    ]\n  ],\n  moov: [\n    mvhd: [\n      version: 0,\n      fields: [\n        version: {:uint8, [store: :version]},\n        flags: {:uint24, [store: :fo_flags]},\n        creation_time: {:uint32, [when: {:version, [value: 0]}]},\n        creation_time: {:uint64, [when: {:version, [value: 1]}]},\n        modification_time: {:uint32, [when: {:version, [value: 0]}]},\n        modification_time: {:uint64, [when: {:version, [value: 1]}]},\n        timescale: :uint32,\n        duration: {:uint32, [when: {:version, [value: 0]}]},\n        duration: {:uint64, [when: {:version, [value: 1]}]},\n        rate: :fp16d16,\n        volume: :fp8d8,\n        reserved: <<0, 0, 0, 0, 0, 0, 0, 0, 0, 0>>,\n        matrix_value_A: :fp16d16,\n        matrix_value_B: :fp16d16,\n        matrix_value_U: :fp2d30,\n        matrix_value_C: :fp16d16,\n        matrix_value_D: :fp16d16,\n        matrix_value_V: :fp2d30,\n        matrix_value_X: :fp16d16,\n        matrix_value_Y: :fp16d16,\n        matrix_value_W: :fp2d30,\n        quicktime_preview_time: :uint32,\n        quicktime_preview_duration: :uint32,\n        quicktime_poster_time: :uint32,\n        quicktime_selection_time: :uint32,\n        quicktime_selection_duration: :uint32,\n        quicktime_current_time: :uint32,\n        next_track_id: :uint32\n      ]\n    ],\n    trak: [\n      tkhd: [\n        version: 0,\n        fields: [\n          version: {:uint8, [store: :version]},\n          flags: {:uint24, [store: :fo_flags]},\n          creation_time: {:uint32, [when: {:version, [value: 0]}]},\n          creation_time: {:uint64, [when: {:version, [value: 1]}]},\n          modification_time: {:uint32, [when: {:version, [value: 0]}]},\n          modification_time: {:uint64, [when: {:version, [value: 1]}]},\n          track_id: :uint32,\n          reserved: <<0, 0, 0, 0>>,\n          duration: {:uint32, [when: {:version, [value: 0]}]},\n          duration: {:uint64, [when: {:version, [value: 1]}]},\n          reserved: <<0, 0, 0, 0, 0, 0, 0, 0>>,\n          layer: :int16,\n          alternate_group: :int16,\n          volume: :fp8d8,\n          reserved: <<0, 0>>,\n          matrix_value_A: :fp16d16,\n          matrix_value_B: :fp16d16,\n          matrix_value_U: :fp2d30,\n          matrix_value_C: :fp16d16,\n          matrix_value_D: :fp16d16,\n          matrix_value_V: :fp2d30,\n          matrix_value_X: :fp16d16,\n          matrix_value_Y: :fp16d16,\n          matrix_value_W: :fp2d30,\n          width: :fp16d16,\n          height: :fp16d16\n        ]\n      ],\n      mdia: [\n        mdhd: [\n          version: 0,\n          fields: [\n            version: {:uint8, [store: :version]},\n            flags: {:uint24, [store: :fo_flags]},\n            creation_time: {:uint32, [when: {:version, [value: 0]}]},\n            creation_time: {:uint64, [when: {:version, [value: 1]}]},\n            modification_time: {:uint32, [when: {:version, [value: 0]}]},\n            modification_time: {:uint64, [when: {:version, [value: 1]}]},\n            timescale: :uint32,\n            duration: {:uint32, [when: {:version, [value: 0]}]},\n            duration: {:uint64, [when: {:version, [value: 1]}]},\n            reserved: <<0::size(1)>>,\n            language: :uint15,\n            reserved: <<0, 0>>\n          ]\n        ],\n        hdlr: [\n          version: 0,\n          fields: [\n            version: {:uint8, [store: :version]},\n            flags: {:uint24, [store: :fo_flags]},\n            reserved: <<0, 0, 0, 0>>,\n            handler_type: :str32,\n            reserved: <<0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0>>,\n            name: :str\n          ]\n        ],\n        minf: [\n          vmhd: [\n            version: 0,\n            fields: [\n              version: {:uint8, [store: :version]},\n              flags: {:uint24, [store: :fo_flags]},\n              graphics_mode: :uint16,\n              opcolor: :uint48\n            ]\n          ],\n          smhd: [\n            version: 0,\n            fields: [\n              version: {:uint8, [store: :version]},\n              flags: {:uint24, [store: :fo_flags]},\n              balance: :fp8d8,\n              reserved: <<0, 0>>\n            ]\n          ],\n          dinf: [\n            dref: [\n              version: 0,\n              fields: [\n                version: {:uint8, [store: :version]},\n                flags: {:uint24, [store: :fo_flags]},\n                entry_count: :uint32\n              ],\n              url: [\n                version: 0,\n                fields: [\n                  version: {:uint8, [store: :version]},\n                  flags: {:uint24, [store: :fo_flags]}\n                ]\n              ]\n            ]\n          ],\n          stbl: [\n            stsd: [\n              version: 0,\n              fields: [\n                version: {:uint8, [store: :version]},\n                flags: {:uint24, [store: :fo_flags]},\n                entry_count: :uint32\n              ],\n              avc1: [\n                version: 0,\n                fields: [\n                  version: {:uint8, [store: :version]},\n                  flags: {:uint24, [store: :fo_flags]},\n                  num_of_entries: :uint32,\n                  reserved: <<0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0>>,\n                  width: :uint16,\n                  height: :uint16,\n                  horizresolution: :fp16d16,\n                  vertresolution: :fp16d16,\n                  reserved: <<0, 0, 0, 0>>,\n                  frame_count: :uint16,\n                  compressor_name: :str256,\n                  depth: :uint16,\n                  reserved: <<255, 255>>\n                ],\n                avcC: [black_box?: true],\n                pasp: [fields: [h_spacing: :uint32, v_spacing: :uint32]]\n              ],\n              avc3: [\n                version: 0,\n                fields: [\n                  version: {:uint8, [store: :version]},\n                  flags: {:uint24, [store: :fo_flags]},\n                  num_of_entries: :uint32,\n                  reserved: <<0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0>>,\n                  width: :uint16,\n                  height: :uint16,\n                  horizresolution: :fp16d16,\n                  vertresolution: :fp16d16,\n                  reserved: <<0, 0, 0, 0>>,\n                  frame_count: :uint16,\n                  compressor_name: :str256,\n                  depth: :uint16,\n                  reserved: <<255, 255>>\n                ],\n                avcC: [black_box?: true],\n                pasp: [fields: [h_spacing: :uint32, v_spacing: :uint32]]\n              ],\n              hvc1: [\n                version: 0,\n                fields: [\n                  version: {:uint8, [store: :version]},\n                  flags: {:uint24, [store: :fo_flags]},\n                  num_of_entries: :uint32,\n                  reserved: <<0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0>>,\n                  width: :uint16,\n                  height: :uint16,\n                  horizresolution: :fp16d16,\n                  vertresolution: :fp16d16,\n                  reserved: <<0, 0, 0, 0>>,\n                  frame_count: :uint16,\n                  compressor_name: :str256,\n                  depth: :uint16,\n                  reserved: <<255, 255>>\n                ],\n                hvcC: [black_box?: true],\n                pasp: [fields: [h_spacing: :uint32, v_spacing: :uint32]]\n              ],\n              hev1: [\n                version: 0,\n                fields: [\n                  version: {:uint8, [store: :version]},\n                  flags: {:uint24, [store: :fo_flags]},\n                  num_of_entries: :uint32,\n                  reserved: <<0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0>>,\n                  width: :uint16,\n                  height: :uint16,\n                  horizresolution: :fp16d16,\n                  vertresolution: :fp16d16,\n                  reserved: <<0, 0, 0, 0>>,\n                  frame_count: :uint16,\n                  compressor_name: :str256,\n                  depth: :uint16,\n                  reserved: <<255, 255>>\n                ],\n                hvcC: [black_box?: true],\n                pasp: [fields: [h_spacing: :uint32, v_spacing: :uint32]]\n              ],\n              mp4a: [\n                fields: [\n                  reserved: <<0, 0, 0, 0, 0, 0>>,\n                  data_reference_index: :uint16,\n                  encoding_version: :uint16,\n                  encoding_revision: :uint16,\n                  encoding_vendor: :uint32,\n                  channel_count: :uint16,\n                  sample_size: :uint16,\n                  compression_id: :uint16,\n                  packet_size: :uint16,\n                  sample_rate: :fp16d16\n                ],\n                esds: [\n                  version: 0,\n                  fields: [\n                    version: {:uint8, [store: :version]},\n                    flags: {:uint24, [store: :fo_flags]},\n                    elementary_stream_descriptor: :bin\n                  ]\n                ]\n              ],\n              Opus: [\n                version: 0,\n                fields: [\n                  reserved: <<0, 0, 0, 0, 0, 0>>,\n                  data_reference_index: :uint16,\n                  reserved: <<0, 0, 0, 0, 0, 0, 0, 0>>,\n                  channel_count: :uint16,\n                  sample_size: :uint16,\n                  reserved: <<0, 0>>,\n                  reserved: <<0, 0>>,\n                  sample_rate: :uint32\n                ],\n                dOps: [\n                  version: 0,\n                  fields: [\n                    version: :uint8,\n                    output_channel_count: :uint8,\n                    pre_skip: :uint16,\n                    input_sample_rate: :uint32,\n                    output_gain: :int16,\n                    channel_mapping_family: :uint8\n                  ]\n                ]\n              ]\n            ],\n            stts: [\n              version: 0,\n              fields: [\n                version: {:uint8, [store: :version]},\n                flags: {:uint24, [store: :fo_flags]},\n                entry_count: :uint32,\n                entry_list: {:list,\n                 [sample_count: :uint32, sample_delta: :uint32]}\n              ]\n            ],\n            stss: [\n              version: 0,\n              fields: [\n                version: {:uint8, [store: :version]},\n                flags: {:uint24, [store: :fo_flags]},\n                entry_count: :uint32,\n                entry_list: {:list, [sample_number: :uint32]}\n              ]\n            ],\n            ctts: [\n              version: 0,\n              fields: [\n                version: {:uint8, [store: :version]},\n                flags: {:uint24, [store: :fo_flags]},\n                entry_count: :uint32,\n                entry_list: {:list,\n                 [sample_count: :uint32, sample_composition_offset: :uint32]}\n              ]\n            ],\n            stsc: [\n              version: 0,\n              fields: [\n                version: {:uint8, [store: :version]},\n                flags: {:uint24, [store: :fo_flags]},\n                entry_count: :uint32,\n                entry_list: {:list,\n                 [\n                   first_chunk: :uint32,\n                   samples_per_chunk: :uint32,\n                   sample_description_index: :uint32\n                 ]}\n              ]\n            ],\n            stsz: [\n              version: 0,\n              fields: [\n                version: {:uint8, [store: :version]},\n                flags: {:uint24, [store: :fo_flags]},\n                sample_size: :uint32,\n                sample_count: :uint32,\n                entry_list: {:list, [entry_size: :uint32]}\n              ]\n            ],\n            stco: [\n              version: 0,\n              fields: [\n                version: {:uint8, [store: :version]},\n                flags: {:uint24, [store: :fo_flags]},\n                entry_count: :uint32,\n                entry_list: {:list, [chunk_offset: :uint32]}\n              ]\n            ],\n            co64: [\n              version: 0,\n              fields: [\n                version: {:uint8, [store: :version]},\n                flags: {:uint24, [store: :fo_flags]},\n                entry_count: :uint32,\n                entry_list: {:list, [chunk_offset: :uint64]}\n              ]\n            ]\n          ]\n        ]\n      ]\n    ],\n    mvex: [\n      mehd: [\n        version: 0,\n        fields: [\n          version: {:uint8, [store: :version]},\n          flags: {:uint24, [store: :fo_flags]},\n          fragment_duration: {:uint32, [when: {:version, [value: 0]}]},\n          fragment_duration: {:uint64, [when: {:version, [value: 1]}]}\n        ]\n      ],\n      trex: [\n        version: 0,\n        fields: [\n          version: {:uint8, [store: :version]},\n          flags: {:uint24, [store: :fo_flags]},\n          track_id: :uint32,\n          default_sample_description_index: :uint32,\n          default_sample_duration: :uint32,\n          default_sample_size: :uint32,\n          default_sample_flags: :uint32\n        ]\n      ]\n    ]\n  ],\n  styp: [\n    fields: [\n      major_brand: :str32,\n      major_brand_version: :uint32,\n      compatible_brands: {:list, :str32}\n    ]\n  ],\n  sidx: [\n    version: 1,\n    fields: [\n      version: {:uint8, [store: :version]},\n      flags: {:uint24, [store: :fo_flags]},\n      reference_id: :uint32,\n      timescale: :uint32,\n      earliest_presentation_time: :uint64,\n      first_offset: :uint64,\n      reserved: <<0, 0>>,\n      reference_count: :uint16,\n      reference_type: :bin1,\n      referenced_size: :uint31,\n      subsegment_duration: :uint32,\n      starts_with_sap: :bin1,\n      sap_type: :uint3,\n      sap_delta_time: :uint28\n    ]\n  ],\n  moof: [\n    mfhd: [\n      version: 0,\n      fields: [\n        version: {:uint8, [store: :version]},\n        flags: {:uint24, [store: :fo_flags]},\n        sequence_number: :uint32\n      ]\n    ],\n    traf: [\n      tfhd: [\n        version: 0,\n        fields: [\n          version: {:uint8, [store: :version]},\n          flags: {:uint24, [store: :fo_flags]},\n          track_id: :uint32,\n          base_data_offset: :uint64,\n          default_sample_description_index: {:uint32,\n           [when: {:fo_flags, [mask: 2]}]},\n          default_sample_duration: {:uint32, [when: {:fo_flags, [mask: 8]}]},\n          default_sample_size: {:uint32, [when: {:fo_flags, [mask: 16]}]},\n          default_sample_flags: {:uint32, [when: {:fo_flags, [mask: 32]}]}\n        ]\n      ],\n      tfdt: [\n        version: 1,\n        fields: [\n          version: {:uint8, [store: :version]},\n          flags: {:uint24, [store: :fo_flags]},\n          base_media_decode_time: :uint64\n        ]\n      ],\n      trun: [\n        version: 0,\n        fields: [\n          version: {:uint8, [store: :version]},\n          flags: {:uint24, [store: :fo_flags]},\n          sample_count: :uint32,\n          data_offset: {:uint32, [when: {:fo_flags, [mask: 1]}]},\n          first_sample_flags: {:bin32, [when: {:fo_flags, [mask: 4]}]},\n          samples: {:list,\n           [\n             sample_duration: {:uint32, [when: {:fo_flags, [mask: 256]}]},\n             sample_size: {:uint32, [when: {:fo_flags, [mask: 512]}]},\n             sample_flags: {:bin32, [when: {:fo_flags, [mask: 1024]}]},\n             sample_composition_offset: {:uint32,\n              [when: {:fo_flags, [mask: 2048]}]}\n           ]}\n        ]\n      ]\n    ]\n  ],\n  mdat: [black_box?: true]\n]\n```","ref":"ExMP4.Container.Schema.html#t:schema_def_t/0"},{"type":"type","title":"ExMP4.Container.Schema.t/0","doc":"The schema of MP4 structure.\n\nAn MP4 file consists of boxes, that all have the same header and different internal\nstructures. Boxes can be nested with one another.\n\nEach box has at most 4-letter name and may have the following parameters:\n- `black_box?` - if true, the box content is unspecified and is treated as an opaque\nbinary. Defaults to false.\n- `version` - the box version. Versions usually differ by the sizes of particular fields.\n- `fields` - a list of key-value parameters\n- `children` - the nested boxes","ref":"ExMP4.Container.Schema.html#t:t/0"},{"type":"module","title":"ExMP4.Track","doc":"A struct describing an MP4 track.","ref":"ExMP4.Track.html"},{"type":"function","title":"ExMP4.Track.add_fragment/2","doc":"","ref":"ExMP4.Track.html#add_fragment/2"},{"type":"function","title":"ExMP4.Track.bitrate/1","doc":"Get the bitrate of the track in `bps` (bit per second)","ref":"ExMP4.Track.html#bitrate/1"},{"type":"function","title":"ExMP4.Track.duration/2","doc":"Get the duration of the track.","ref":"ExMP4.Track.html#duration/2"},{"type":"function","title":"ExMP4.Track.fps/1","doc":"Get the fps (frames per second) of the video track.","ref":"ExMP4.Track.html#fps/1"},{"type":"function","title":"ExMP4.Track.new/1","doc":"Create a new track","ref":"ExMP4.Track.html#new/1"},{"type":"type","title":"ExMP4.Track.box/0","doc":"","ref":"ExMP4.Track.html#t:box/0"},{"type":"type","title":"ExMP4.Track.id/0","doc":"","ref":"ExMP4.Track.html#t:id/0"},{"type":"type","title":"ExMP4.Track.t/0","doc":"Struct describing an mp4 track.\n\nThe public fields are:\n  * `:id` - the track id\n  * `:type` - the type of the media of the track\n  * `:media` - the codec used to encode the media.\n  * `:media_tag` - the box `name` to use for the `:media`.\n\n      This field is used to indicate the layout of some codec specific data, take for example `H264`,\n      `avc1` indicates that parameter sets are included in the sample description and removed from\n      the samples themselves.\n  * `:priv_data` - private data specific to the contained `:media`.\n  * `:duration` - the duration of the track in `:timescale` units.\n  * `:timescale` - the timescale used for the track.\n  * `:width` - the width of the video frames.\n  * `:height` - the height of the video frames.\n  * `:sample_rate` - the sample rate of audio samples.\n  * `:channels` - the number of audio channels.\n  * `:sample_count` - the total count of samples.","ref":"ExMP4.Track.html#t:t/0"},{"type":"module","title":"ExMP4.Track.Fragment","doc":"Module representing a movie fragment (`moof`) of a single track.","ref":"ExMP4.Track.Fragment.html"},{"type":"function","title":"ExMP4.Track.Fragment.add_run/2","doc":"","ref":"ExMP4.Track.Fragment.html#add_run/2"},{"type":"function","title":"ExMP4.Track.Fragment.duration/2","doc":"","ref":"ExMP4.Track.Fragment.html#duration/2"},{"type":"function","title":"ExMP4.Track.Fragment.flush/1","doc":"","ref":"ExMP4.Track.Fragment.html#flush/1"},{"type":"function","title":"ExMP4.Track.Fragment.new/2","doc":"","ref":"ExMP4.Track.Fragment.html#new/2"},{"type":"function","title":"ExMP4.Track.Fragment.store_sample/2","doc":"","ref":"ExMP4.Track.Fragment.html#store_sample/2"},{"type":"function","title":"ExMP4.Track.Fragment.total_samples/1","doc":"","ref":"ExMP4.Track.Fragment.html#total_samples/1"},{"type":"function","title":"ExMP4.Track.Fragment.total_size/2","doc":"","ref":"ExMP4.Track.Fragment.html#total_size/2"},{"type":"function","title":"ExMP4.Track.Fragment.update_base_data_offset/2","doc":"","ref":"ExMP4.Track.Fragment.html#update_base_data_offset/2"},{"type":"type","title":"ExMP4.Track.Fragment.t/0","doc":"","ref":"ExMP4.Track.Fragment.html#t:t/0"},{"type":"module","title":"ExMP4.Track.Fragment.Run","doc":"A struct representing a run (`trun` box) in a fragment.","ref":"ExMP4.Track.Fragment.Run.html"},{"type":"function","title":"ExMP4.Track.Fragment.Run.sample_metadata/1","doc":"","ref":"ExMP4.Track.Fragment.Run.html#sample_metadata/1"},{"type":"function","title":"ExMP4.Track.Fragment.Run.store_sample/2","doc":"","ref":"ExMP4.Track.Fragment.Run.html#store_sample/2"},{"type":"type","title":"ExMP4.Track.Fragment.Run.t/0","doc":"","ref":"ExMP4.Track.Fragment.Run.html#t:t/0"},{"type":"module","title":"ExMP4.Track.FragmentedSampleTable","doc":"A module that defines a structure for a fragmented sample table.","ref":"ExMP4.Track.FragmentedSampleTable.html"},{"type":"function","title":"ExMP4.Track.FragmentedSampleTable.add_fragment/2","doc":"Add a fragment to the sample table.","ref":"ExMP4.Track.FragmentedSampleTable.html#add_fragment/2"},{"type":"type","title":"ExMP4.Track.FragmentedSampleTable.t/0","doc":"","ref":"ExMP4.Track.FragmentedSampleTable.html#t:t/0"},{"type":"module","title":"ExMP4.Track.SampleTable","doc":"A module that defines a structure and functions allowing to store samples,\nassemble them into chunks and flush when needed. Its public functions take\ncare of recording information required to build a sample table.","ref":"ExMP4.Track.SampleTable.html"},{"type":"function","title":"ExMP4.Track.SampleTable.chunk_duration/1","doc":"Get the current chunk duration.\n\nSamples are added and buffered in the sample table into chunks, once a chunk duration\nis reached, it can be flushed using `flush_chunk/2`.","ref":"ExMP4.Track.SampleTable.html#chunk_duration/1"},{"type":"function","title":"ExMP4.Track.SampleTable.flush_chunk/2","doc":"Flush the current chunk.\n\nFlushing update the internal structure of the sample table and assemble\nthe samples payload into a binary that can be stored.","ref":"ExMP4.Track.SampleTable.html#flush_chunk/2"},{"type":"function","title":"ExMP4.Track.SampleTable.store_sample/2","doc":"Store a new sample.","ref":"ExMP4.Track.SampleTable.html#store_sample/2"},{"type":"function","title":"ExMP4.Track.SampleTable.total_size/1","doc":"Get the total size of the samples in the sample table.","ref":"ExMP4.Track.SampleTable.html#total_size/1"},{"type":"type","title":"ExMP4.Track.SampleTable.t/0","doc":"","ref":"ExMP4.Track.SampleTable.html#t:t/0"},{"type":"module","title":"ExMP4.Codec.Avc","doc":"Module responsible for parsing and generating AVC Configuration Record.","ref":"ExMP4.Codec.Avc.html"},{"type":"function","title":"ExMP4.Codec.Avc.new/3","doc":"Creates a new decoder configuration record from parameter sets.","ref":"ExMP4.Codec.Avc.html#new/3"},{"type":"function","title":"ExMP4.Codec.Avc.parse/1","doc":"Parses the DCR.","ref":"ExMP4.Codec.Avc.html#parse/1"},{"type":"function","title":"ExMP4.Codec.Avc.serialize/1","doc":"Serializes decoder configuration record.","ref":"ExMP4.Codec.Avc.html#serialize/1"},{"type":"type","title":"ExMP4.Codec.Avc.new_opts/0","doc":"","ref":"ExMP4.Codec.Avc.html#t:new_opts/0"},{"type":"type","title":"ExMP4.Codec.Avc.t/0","doc":"Structure representing the Decoder Configuartion Record","ref":"ExMP4.Codec.Avc.html#t:t/0"},{"type":"module","title":"ExMP4.Codec.Hevc","doc":"Module responsible for parsing and generating HEVC Configuration Record.","ref":"ExMP4.Codec.Hevc.html"},{"type":"function","title":"ExMP4.Codec.Hevc.parse/1","doc":"Parses the DCR.","ref":"ExMP4.Codec.Hevc.html#parse/1"},{"type":"function","title":"ExMP4.Codec.Hevc.serialize/1","doc":"Serializes a DCR","ref":"ExMP4.Codec.Hevc.html#serialize/1"},{"type":"type","title":"ExMP4.Codec.Hevc.t/0","doc":"Structure representing the Decoder Configuartion Record","ref":"ExMP4.Codec.Hevc.html#t:t/0"},{"type":"behaviour","title":"ExMP4.DataReader","doc":"A behaviour module for implementing mp4 data reader.","ref":"ExMP4.DataReader.html"},{"type":"callback","title":"ExMP4.DataReader.close/1","doc":"Close the input stream.","ref":"ExMP4.DataReader.html#c:close/1"},{"type":"callback","title":"ExMP4.DataReader.open/1","doc":"Open the provided input in read mode\n\nThe returned `state` will be the first argument on the other callbacks","ref":"ExMP4.DataReader.html#c:open/1"},{"type":"callback","title":"ExMP4.DataReader.read/3","doc":"Read the specified amount of bytes from the input.\n\nAn optional location may be provided to seek into the input.","ref":"ExMP4.DataReader.html#c:read/3"},{"type":"callback","title":"ExMP4.DataReader.seek/2","doc":"Seek to the provided location in the input stream","ref":"ExMP4.DataReader.html#c:seek/2"},{"type":"type","title":"ExMP4.DataReader.location/0","doc":"","ref":"ExMP4.DataReader.html#t:location/0"},{"type":"type","title":"ExMP4.DataReader.reason/0","doc":"","ref":"ExMP4.DataReader.html#t:reason/0"},{"type":"type","title":"ExMP4.DataReader.state/0","doc":"","ref":"ExMP4.DataReader.html#t:state/0"},{"type":"module","title":"ExMP4.DataReader.File","doc":"Implementation of the `ExMP4.DataReader` behaviour using filesystem.","ref":"ExMP4.DataReader.File.html"},{"type":"behaviour","title":"ExMP4.DataWriter","doc":"A behaviour module for implementing mp4 data writer.","ref":"ExMP4.DataWriter.html"},{"type":"callback","title":"ExMP4.DataWriter.close/1","doc":"Close the output.","ref":"ExMP4.DataWriter.html#c:close/1"},{"type":"callback","title":"ExMP4.DataWriter.open/1","doc":"Initialize the output.\n\nThe returned `state` will be the first argument on the other callbacks.","ref":"ExMP4.DataWriter.html#c:open/1"},{"type":"callback","title":"ExMP4.DataWriter.write/4","doc":"Invoked to handle writing an ISOBMFF box.\n\nThe `data` may be a binary or an IO list.\n\n`location` if provided is the location in the output where to store the box.\nIf `insert?` provided (defaults to `false`), the data should be inserted into\nthat position and not overwriting existing data.","ref":"ExMP4.DataWriter.html#c:write/4"},{"type":"type","title":"ExMP4.DataWriter.location/0","doc":"","ref":"ExMP4.DataWriter.html#t:location/0"},{"type":"type","title":"ExMP4.DataWriter.state/0","doc":"","ref":"ExMP4.DataWriter.html#t:state/0"},{"type":"module","title":"ExMP4.DataWriter.File","doc":"Implementation of the `ExMP4.DataWriter` behaviour using filesystem (single file).","ref":"ExMP4.DataWriter.File.html"},{"type":"behaviour","title":"ExMP4.FragDataWriter","doc":"A behaviour module for implementing fragmented mp4 data writer.","ref":"ExMP4.FragDataWriter.html"},{"type":"callback","title":"ExMP4.FragDataWriter.close/1","doc":"Close the output.","ref":"ExMP4.FragDataWriter.html#c:close/1"},{"type":"callback","title":"ExMP4.FragDataWriter.open/1","doc":"Initialize the output.\n\nThe returned `state` will be the first argument on the other callbacks.","ref":"ExMP4.FragDataWriter.html#c:open/1"},{"type":"callback","title":"ExMP4.FragDataWriter.write/3","doc":"Invoked to handle writing while seeking into the file.\n\nThis is an optional callback, only called by the `ExMP4.FWriter` to\nupdate the fragments duration in case `mehd` is present.","ref":"ExMP4.FragDataWriter.html#c:write/3"},{"type":"callback","title":"ExMP4.FragDataWriter.write_fragment/2","doc":"Invoked to handle writing the whole fragment (`sidx` [optional] + `moof` + `mdat`).","ref":"ExMP4.FragDataWriter.html#c:write_fragment/2"},{"type":"callback","title":"ExMP4.FragDataWriter.write_init_header/2","doc":"Invoked to handle writing media header initialization.","ref":"ExMP4.FragDataWriter.html#c:write_init_header/2"},{"type":"type","title":"ExMP4.FragDataWriter.location/0","doc":"","ref":"ExMP4.FragDataWriter.html#t:location/0"},{"type":"type","title":"ExMP4.FragDataWriter.state/0","doc":"","ref":"ExMP4.FragDataWriter.html#t:state/0"},{"type":"module","title":"ExMP4.FragDataWriter.File","doc":"Implementation of the `ExMP4.FragDataWriter` behaviour using filesystem (single file).","ref":"ExMP4.FragDataWriter.File.html"},{"type":"module","title":"ExMP4.Box.FileType","doc":"A module containing a function for assembling an MPEG-4 file type box.\n\nThe file type box (`ftyp` atom) is a top-level box that contains specifications\nand compatibility information that media players can use to correctly interpret\nan MPEG-4 container.","ref":"ExMP4.Box.FileType.html"},{"type":"function","title":"ExMP4.Box.FileType.assemble/3","doc":"","ref":"ExMP4.Box.FileType.html#assemble/3"},{"type":"module","title":"ExMP4.Box.MediaData","doc":"A module containing a function for assembling an MPEG-4 media data box.\n\nThe media data box (`mdat` atom) is a top-level box that contains actual media data\n(e.g. encoded video frames or audio samples). The data is logically divided into so\ncalled \"chunks\" that consist of \"samples\". There are no assumptions made about chunks\narrangement or sizes in the MPEG-4 specification and chunks belonging to different tracks\ncan (and should) be interleaved.","ref":"ExMP4.Box.MediaData.html"},{"type":"function","title":"ExMP4.Box.MediaData.assemble/1","doc":"","ref":"ExMP4.Box.MediaData.html#assemble/1"},{"type":"module","title":"ExMP4.Box.Movie","doc":"A module providing a function assembling an MPEG-4 movie box.\n\nThe movie box (`moov`) is a top-level box that contains information about\na presentation as a whole. It consists of:\n\n  * exactly one movie header (`mvhd` atom)\n\n    The movie header contains media-independent data, such as the\n    number of tracks, volume, duration or timescale (presentation-wide).\n\n  * one or more track box (`trak` atom)\n\n  * zero or one movie extends box (`mvex` atom)","ref":"ExMP4.Box.Movie.html"},{"type":"function","title":"ExMP4.Box.Movie.assemble/3","doc":"Assembles a list of tracks into a `moov` box.","ref":"ExMP4.Box.Movie.html#assemble/3"},{"type":"function","title":"ExMP4.Box.Movie.unpack/1","doc":"Deserializes and unpacks a `moov` box data.\n\nThe return type is a tuple with the first element as the movie header and the\nsecond element as a list of tracks.","ref":"ExMP4.Box.Movie.html#unpack/1"},{"type":"type","title":"ExMP4.Box.Movie.movie_header/0","doc":"","ref":"ExMP4.Box.Movie.html#t:movie_header/0"},{"type":"module","title":"ExMP4.Box.MovieExtendsBox","doc":"The module provides a function that assembles an MPEG-4 movie extends box (`mvex` atom).\n\nThe movie extends box provides information about movie fragment boxes in case when\nmedia data is fragmented (for example in CMAF). It has to contain as many track\nextends box (`trex` atoms) as there are tracks in the movie box.\n\nFor more information about the movie extends box, refer to [ISO/IEC 14496-12](https://www.iso.org/standard/74428.html).","ref":"ExMP4.Box.MovieExtendsBox.html"},{"type":"function","title":"ExMP4.Box.MovieExtendsBox.assemble/2","doc":"","ref":"ExMP4.Box.MovieExtendsBox.html#assemble/2"},{"type":"module","title":"ExMP4.Box.MovieFragment","doc":"A module containing a function for assembling an MPEG-4 movie fragment (moof) box.","ref":"ExMP4.Box.MovieFragment.html"},{"type":"function","title":"ExMP4.Box.MovieFragment.assemble/2","doc":"Creates a new `moof` box from the `ExMP4.Track.Fragment` fragments.","ref":"ExMP4.Box.MovieFragment.html#assemble/2"},{"type":"function","title":"ExMP4.Box.MovieFragment.unpack/1","doc":"Unpacks a `moof` box into a list of `ExMP4.Track.Fragment` fragments.","ref":"ExMP4.Box.MovieFragment.html#unpack/1"},{"type":"function","title":"ExMP4.Box.MovieFragment.update_base_data_offsets/2","doc":"","ref":"ExMP4.Box.MovieFragment.html#update_base_data_offsets/2"},{"type":"module","title":"ExMP4.Box.Track","doc":"A module containing a set of utilities for assembling an MPEG-4 track box.\n\nThe track box (`trak` atom) describes a single track of a presentation. This description includes\ninformation like its timescale, duration, volume, media-specific data (media handlers, sample\ndescriptions) as well as a sample table, which allows media players to find and interpret\ntrack's data in the media data box.\n\nFor more information about the track box, refer to [ISO/IEC 14496-12](https://www.iso.org/standard/74428.html).","ref":"ExMP4.Box.Track.html"},{"type":"function","title":"ExMP4.Box.Track.assemble/1","doc":"","ref":"ExMP4.Box.Track.html#assemble/1"},{"type":"function","title":"ExMP4.Box.Track.unpack/1","doc":"","ref":"ExMP4.Box.Track.html#unpack/1"},{"type":"extras","title":"ExMP4","doc":"# ExMP4\n\n[![Hex.pm](https://img.shields.io/hexpm/v/ex_mp4.svg)](https://hex.pm/packages/ex_mp4)\n[![API Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/ex_mp4)\n\n`ISO-MP4` reader and writer.\n\nThis package contains MPEG-4 specifications defined in parts:\n* ISO/IEC 14496-12 - ISO Base Media File Format (QuickTime, MPEG-4, etc)\n* ISO/IEC 14496-14 - MP4 file format\n\nThis package is an alternative to and inspired from [membrane_mp4_plugin](https://github.com/membraneframework/membrane_mp4_plugin). This package differs from the `membrane_mp4_plugin` in that it allows to manipulate mp4 files without the usage of membrane pipelines.","ref":"readme.html"},{"type":"extras","title":"Installation - ExMP4","doc":"The package can be installed by adding `ex_mp4` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ex_mp4, \"~> 0.4.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"API - ExMP4","doc":"The API is not yet stable, so breaking changes may occur when upgrading minor version.","ref":"readme.html#api"},{"type":"extras","title":"Usage - ExMP4","doc":"check the `examples` folder for usage.","ref":"readme.html#usage"},{"type":"extras","title":"Video Track Copier","doc":"# Video Track Copier\n\n```elixir\nMix.install([:ex_mp4, {:kino, \"~> 0.11.0\"}])\n```","ref":"copy_track.html"},{"type":"extras","title":"Track Copier - Video Track Copier","doc":"In this example, we'll copy the video track from an input MP4 file to another file.\n\n<!-- livebook:{\"break_markdown\":true} -->\n\nFirst let's add a module that'll copy the video track from a file to another one\n\n```elixir\ndefmodule TrackCopier do\n  require Logger\n\n  alias ExMP4.{Reader, Writer}\n\n  def copy(source, dest) do\n    reader = Reader.new!(source)\n    writer = Writer.new!(dest)\n\n    Logger.info(\"\"\"\n    Reader info: ===============\n      Path: #{inspect(source)}\n      Duration: #{inspect(reader.duration)}\n      Timescale: #{inspect(reader.timescale)}\n      Tracks: #{length(Reader.tracks(reader))}\n    \"\"\")\n\n    video_track =\n      reader\n      |> Reader.tracks()\n      |> Enum.find(&(&1.type == :video))\n\n    writer =\n      writer\n      |> Writer.write_header()\n      |> Writer.add_track(video_track)\n\n    Reader.stream(reader, tracks: [video_track.id])\n    |> Reader.samples(reader)\n    |> Stream.map(&%{&1 | track_id: 1})\n    |> Enum.into(writer)\n    |> Writer.write_trailer()\n  end\nend\n```\n\nNext we create a form that accepts the source path and the destination of the new MP4 file.\n\n```elixir\nform =\n  Kino.Control.form(\n    [\n      source_path: Kino.Input.text(\"Source Path\"),\n      dest_path: Kino.Input.text(\"Destination Path\")\n    ],\n    submit: \"Submit\"\n  )\n\nKino.listen(form, fn %{data: data} ->\n  :ok = TrackCopier.copy(data.source_path, data.dest_path)\n  IO.inspect(\"Done copying\")\nend)\n\nform\n```","ref":"copy_track.html#track-copier"},{"type":"extras","title":"Crop Video","doc":"# Crop Video\n\n```elixir\nMix.install([:ex_mp4, {:kino, \"~> 0.11.0\"}])\n```","ref":"crop.html"},{"type":"extras","title":"Crop Video - Crop Video","doc":"In this example, we'll crop an mp4 file by getting samples between two timestamps. The goal of this example is to show how to seek in a file.\n\n<!-- livebook:{\"break_markdown\":true} -->\n\nFirst let's add a module that'll do the seeking and writing. The process is as follows:","ref":"crop.html#crop-video"},{"type":"extras","title":"Seek to a keyframe - Crop Video","doc":"Since we cannot just start writing video samples from an arbitrary position, we need first to lookup the nearest key frame before the requested time to ensure that the video track is decodable when playing.","ref":"crop.html#seek-to-a-keyframe"},{"type":"extras","title":"Get track ranges - Crop Video","doc":"From the last step, we'll get the start time and end time in the video track timescale, we'll convert this times to each track timescale. So that yields a map with the track id as a key and a tuple of start time and end time as a value.","ref":"crop.html#get-track-ranges"},{"type":"extras","title":"Stream the samples metadata - Crop Video","doc":"Next we stream the samples' metadata and filter out samples that are not in the range calculated from the last step","ref":"crop.html#stream-the-samples-metadata"},{"type":"extras","title":"Write the samples - Crop Video","doc":"Last step would be to create a writer, add the tracks and store the filtered samples.\n\n```elixir\ndefmodule VideoCropper do\n  require Logger\n\n  alias ExMP4.{Helper, Reader, Writer}\n\n  def crop(video_path, dest_path, start_time, duration) do\n    reader = Reader.new!(video_path)\n\n    {start_time, end_time, timescale} = get_real_range(reader, start_time, duration)\n    track_ranges = get_track_ranges(reader, start_time, end_time, timescale)\n\n    tracks = Reader.tracks(reader) |> Enum.sort_by(& &1.id)\n\n    writer =\n      Writer.new!(dest_path)\n      |> Writer.write_header()\n      |> Writer.add_tracks(tracks)\n\n    Reader.stream(reader)\n    |> Stream.filter(fn metadata ->\n      {s, e} = track_ranges[metadata.track_id]\n      metadata.dts >= s and metadata.dts <= e\n    end)\n    |> Reader.samples(reader)\n    |> Enum.into(writer)\n    |> Writer.write_trailer()\n  end\n\n  defp get_real_range(reader, start_time, duration) do\n    video_track =\n      reader\n      |> Reader.tracks()\n      |> Enum.find(&(&1.type == :video))\n\n    end_time = Helper.timescalify(start_time + duration, :second, video_track.timescale)\n    start_time = Helper.timescalify(start_time, :second, video_track.timescale)\n\n    # fetch the starting point\n    start_time =\n      Reader.stream(reader, tracks: [video_track.id])\n      |> Enum.reduce_while(0, fn metadata, offset ->\n        cond do\n          metadata.dts >= start_time -> {:halt, offset}\n          metadata.sync? -> {:cont, metadata.dts}\n          true -> {:cont, offset}\n        end\n      end)\n\n    {start_time, end_time, video_track.timescale}\n  end\n\n  defp get_track_ranges(reader, start_time, end_time, timescale) do\n    Reader.tracks(reader)\n    |> Enum.map(fn track ->\n      range_start = Helper.timescalify(start_time, timescale, track.timescale)\n      range_end = Helper.timescalify(end_time, timescale, track.timescale)\n\n      {track.id, {range_start, range_end}}\n    end)\n    |> Map.new()\n  end\nend\n```\n\nNext we create a form that accepts the source path and the destination of the new MP4 file.\n\n```elixir\nform =\n  Kino.Control.form(\n    [\n      source_path: Kino.Input.text(\"Source Path\"),\n      dest_path: Kino.Input.text(\"Destination Path\"),\n      start_time: Kino.Input.number(\"Start time\", default: 5),\n      duration: Kino.Input.number(\"Duration\", default: 10)\n    ],\n    submit: \"Submit\"\n  )\n\nKino.listen(form, fn %{data: data} ->\n  :ok = VideoCropper.crop(data.source_path, data.dest_path, data.start_time, data.duration)\n  IO.inspect(\"Done cropping\")\nend)\n\nform\n```","ref":"crop.html#write-the-samples"},{"type":"extras","title":"Convert Progressive MP4 to Fragmented","doc":"# Convert Progressive MP4 to Fragmented\n\n```elixir\nMix.install([\n  :ex_mp4,\n  {:kino, \"~> 0.11.0\"}\n])\n```","ref":"progressive_to_fragmented.html"},{"type":"extras","title":"Progressive MP4 to Fragmented - Convert Progressive MP4 to Fragmented","doc":"In this guide, we'll convert an progressive mp4 to a fragmented one.","ref":"progressive_to_fragmented.html#progressive-mp4-to-fragmented"},{"type":"extras","title":"Inputs - Convert Progressive MP4 to Fragmented","doc":"The inputs are the progressive mp4 file and the a fragment duration in millisecond.","ref":"progressive_to_fragmented.html#inputs"},{"type":"extras","title":"Output - Convert Progressive MP4 to Fragmented","doc":"A fragmented mp4\n\n```elixir\ndefmodule MP4Converter do\n  alias ExMP4.{FWriter, Reader}\n\n  def convert(filename, output, fragment_duration \\\\ 4_000) do\n    reader = Reader.new!(filename)\n    tracks = Reader.tracks(reader) |> Enum.sort_by(& &1.id)\n\n    movie_duration =\n      ExMP4.Helper.timescalify(reader.duration, reader.timescale, ExMP4.movie_timescale())\n\n    durations =\n      Map.new(\n        tracks,\n        &{&1.id, ExMP4.Helper.timescalify(fragment_duration, :millisecond, &1.timescale)}\n      )\n\n    curr_durations = Map.new(tracks, &{&1.id, 0})\n\n    writer =\n      FWriter.new!(output, tracks, duration: movie_duration)\n      |> FWriter.create_fragment()\n\n    {writer, _durations} =\n      Reader.stream(reader)\n      |> Reader.samples(reader)\n      |> Enum.reduce({writer, curr_durations}, fn sample, {writer, curr_durations} ->\n        curr_durations = Map.update!(curr_durations, sample.track_id, &(&1 + sample.duration))\n\n        new_fragment? = curr_durations[sample.track_id] >= durations[sample.track_id]\n\n        {writer, durations} =\n          if new_fragment? do\n            curr_durations = Map.new(Map.keys(curr_durations), &{&1, 0})\n            curr_durations = Map.put(curr_durations, sample.track_id, sample.duration)\n            {FWriter.flush_fragment(writer) |> FWriter.create_fragment(), curr_durations}\n          else\n            {writer, curr_durations}\n          end\n\n        {FWriter.write_sample(writer, sample), durations}\n      end)\n\n    writer\n    |> FWriter.flush_fragment()\n    |> FWriter.close()\n  end\nend\n```\n\n```elixir\nform =\n  Kino.Control.form(\n    [\n      source: Kino.Input.text(\"Source Path\"),\n      dest: Kino.Input.text(\"Destination Path\"),\n      duration: Kino.Input.number(\"Duration\", default: 4_000)\n    ],\n    submit: \"Submit\"\n  )\n\nKino.listen(form, fn %{data: data} ->\n  MP4Converter.convert(data.source, data.dest, data.duration)\n  IO.puts(\"Done converting\")\nend)\n\nform\n```","ref":"progressive_to_fragmented.html#output"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright 2018 Software Mansion\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,51,46,48]}}